<?php
/**
 * @file
 * Batch API integration and import functions.
 */

/**
 * Form builder; The import form.
 *
 * @param array $form_state
 *   An associative array containing the current state of the form.
 *
 * @return array
 *   The form structure.
 *
 * @see ucsf_profiles_import_form_submit()
 * @ingroup forms
 */
function ucsf_profiles_import_form(&$form_state) {
  $form['ucsf_profiles']['update'] = array(
    '#type' => 'fieldset',
    '#title' => t('Automatic updates'),
    '#collapsible' => FALSE,
    '#collapsed' => FALSE,
  );
  if (variable_get('ucsf_profiles_cron_last', 0)) {
    $update_time = format_date(variable_get('ucsf_profiles_cron_last', 0));
  }
  else {
    $update_time = t('Never');
  }
  $form['ucsf_profiles']['update']['#description'] = t('Last updated: @date', array('@date' => $update_time));


  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Import publications from UCSF Profiles'),
  );
  return $form;
}

/**
 * Form submission handler for ucsf_profiles_import_form().
 *
 * Triggers the function defining the batch operation.
 *
 * @param array $form
 *   The form being used to edit the module settings.
 *
 * @param array $form_state
 *   An associative array containing the current state of the form.
 */
function ucsf_profiles_import_form_submit($form, &$form_state) {
  ucsf_profiles_batch();
}

/**
 * Callback for the batch operations.
 */
function ucsf_profiles_batch() {
  $employee_id = array_unique(preg_split("/[\s,]+/", variable_get('ucsf_profiles_ids', '')));

  $operations = array();

  $operations[] = array('ucsf_profiles_batch_empty_table', array());

  if (is_array($employee_id)) {
    foreach ($employee_id as $id) {
      $id = (int) $id;
      if (is_int($id)) {
        $operations[] = array('ucsf_profiles_batch_process', array($id));
      }
    }
  }

  $batch = array(
    'title' => t('Importing publications from UCSF Profiles'),
    'operations' => $operations,
    'finished' => 'ucsf_profiles_batch_finished',
    'file' => drupal_get_path('module', 'ucsf_profiles') . '/ucsf_profiles.import.inc',
  );
  batch_set($batch);
  batch_process();
}

/**
 * Truncate table to avoid duplicates and keep the list fresh.
 */
function ucsf_profiles_batch_empty_table(&$context) {
  $table = data_get_table('ucsf_profiles');
  if ($table) {
    $table->handler()->truncate();
  }
}

/**
 * Batch process.
 *
 * @param int $employee_id
 *   Employee ID.
 *
 * @param array $context
 *   Gathers batch context information.
 *   - results: The array of results gathered so far.
 *   - message: A text message displayed in the progress page.
 *   - sandbox
 *   - finished
 *
 * @see http://api.drupal.org/api/drupal/includes--form.inc/group/batch/6
 */
function ucsf_profiles_batch_process($employee_id, &$context) {
  if ($employee_id > 0) {
    // A UCSF Profiles method.
    $person_id = (int) ($employee_id / 10) + 2569307;

    // Prepare the HTTP Request.
    $response = drupal_http_request(
                  'http://profiles.ucsf.edu/api/ProfileService.svc/ProfileSearch',
                  array(
                    'Content-Type' => 'text/xml',
                  ),
                  'POST',
                  '<?xml version="1.0"?><Profiles xmlns="http://connects.profiles.schema/profiles/query" Operation="GetPersonList"><QueryDefinition><PersonID>' . $person_id . '</PersonID></QueryDefinition><OutputOptions SortType="LastFirstName" StartRecord="1" MaxRecords="1000"/></Profiles>',
                  3
                );
    // Make sure response is valid.
    if (($response->code == 200) && ($response->data != '')) {
      if ($xml = json_decode(json_encode(simplexml_load_string($response->data)), TRUE)) {
        $person_id     = $xml['Person']['PersonID'];
        $full_name     = $xml['Person']['Name']['FullName'];
        $first_name    = $xml['Person']['Name']['FirstName'];
        $last_name     = $xml['Person']['Name']['LastName'];
        $profile_url   = $xml['Person']['ProfileURL'];

        $rows = array();

        // Prepare the array.
        if (count($xml['Person']['PublicationList']['Publication']) > 0) {
          foreach ($xml['Person']['PublicationList']['Publication'] as $item) {
            $rows[] = array(
              'ucsf_id' => $employee_id,
              'person_id' => $person_id,
              'full_name' => $full_name,
              'first_name' => $first_name,
              'last_name' => $last_name,
              'profile_url' => $profile_url,
              'pub_id' => $item['PublicationID'],
              'pub_reference' => $item['PublicationReference'],
              'pub_source_name' => $item['PublicationSourceList']['PublicationSource']['@attributes']['Name'],
              'pub_source_url' => $item['PublicationSourceList']['PublicationSource']['@attributes']['URL'],
            );
          }
        }

        // Get the table.
        $table = data_get_table('ucsf_profiles');
        // If this table is not available create the table.
        if (!$table) {
          $table = data_create_table(data_name('ucsf_profiles'));
        }
        // Insert rows.
        foreach ($rows as $i => $row) {
          $table->handler()->save($rows[$i]);
        }
      }

      $context['results'][] = t('Employee ID: @eid, Person ID: @pid, # of publications retrieved: @no', array('@eid' => $employee_id, '@pid' => $person_id, '@no' => count($rows)));
      $context['message'] = t('Downloading for Person ID#@person_id', array('@person_id' => $person_id));
    }
  }
}

/**
 * Finished callback of import batch.
 *
 * Reports the number of XML files that processed, and the number of
 * publications retrieved for each person queried, or an error message
 * explaining what went wrong.
 *
 * @param bool $success
 *   Boolean.
 *
 * @param array $results
 *   The array of results gathered so far by the batch processing.
 *
 * @param array $operations
 *   Callback functions defined by the batch.
 */
function ucsf_profiles_batch_finished($success, $results, $operations) {
  if ($success) {
    $message_title = format_plural(count($results), 'One XML file processed.', '@count XML files processed.');
    $message = theme('item_list', $results, $title = $message_title);

    variable_set('ucsf_profiles_cron_last', $_SERVER['REQUEST_TIME']);
  }
  else {
    // An error occurred.
    // $operations contains the operations that remained unprocessed.
    $error_operation = reset($operations);
    $message = 'An error occurred while processing ' . $error_operation[0] . ' with arguments: ' . print_r($error_operation[0], TRUE);
  }
  drupal_set_message(filter_xss(t('Results: !results', array('!results' => $message))));
}
